unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  LCLtype, Menus, Clipbrd, lazUTF8;

type

  { TForm1 }

  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    Memo1: TMemo;
    MenuItem1: TMenuItem;
    MenuItem10:TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    Open: TOpenDialog;
    Save: TSaveDialog;
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDropFiles(Sender: TObject; const FileNames: array of String);
    procedure FormResize(Sender: TObject);
    procedure Memo1Change(Sender: TObject);
    procedure Memo1KeyPress(Sender: TObject; var Key: char);
    procedure MenuItem10Click(Sender:TObject);
    procedure MenuItem2Click(Sender: TObject);
    procedure MenuItem3Click(Sender: TObject);
    procedure MenuItem4Click(Sender: TObject);
    procedure MenuItem6Click(Sender: TObject);
    procedure MenuItem7Click(Sender: TObject);
    procedure MenuItem8Click(Sender: TObject);
    procedure MenuItem9Click(Sender: TObject);
    procedure FileLoad;
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

type
  Ttable = record
             es, kana : UTF8string;
             size     : smallint;
           end;
const
  ProgramTitle = 'Furigana para Español v1.02';

var
  TextChange : boolean;
  Table      : array[1..300] of Ttable;
  Tmax       : integer;
  WorkFile   : string;

procedure Tform1.FileLoad;
var
  f : text;
  st: string;
begin
  try
    AssignFile(f,WorkFile);
    Reset(f);
    readln(f,st);
    if copy(st,1,3)<>#$EF#$BB#$BF then  {BOM}
      if Application.MessageBox('UTF-8ファイルではありません。それでも読み込みますか？','確認',MB_YESNO)<>IDYES then
      begin
        CloseFile(f);
        exit;
      end;
    Memo1.Text:=copy(st,4,2000); {remove BOM}
    while not eof(f) do
    begin
      readln(f,st);
      Memo1.Append(st);
    end;
  except
    ShowMessage('ファイルアクセスエラー');
    exit;
  end;
  CloseFile(f);
  TextChange:=false;
  Form1.Caption:=ExtractFileName(WorkFile)+' - '+ProgramTitle;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  Memo1.Width:=Form1.Width;
  Memo1.Height:=Form1.Height-20;
end;

procedure TForm1.Memo1Change(Sender: TObject);
begin
  TextChange:=true;
end;

procedure TForm1.Memo1KeyPress(Sender: TObject; var Key: char);
begin
  if key=#$12 then MenuItem9Click(Sender);  {Ctrl-R}
end;

procedure TForm1.MenuItem10Click(Sender:TObject);  {New}
var
  result : integer;
begin
  if TextChange then
  begin
    result:=Application.MessageBox('内容をファイルに保存しますか？','確認',MB_YESNOCANCEL);
    if result=IDYES then MenuItem3Click(Sender)
                    else if result=IDCANCEL then exit; {cancel}
  end;
  WorkFile:=''; {New file}
  Form1.Caption:=ProgramTitle;
  Memo1.Clear;
  TextChange:=false;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  fin : text;
  i   : integer;
  inst: string;
begin
  TextChange:=false;
  WorkFile:='';
  Form1.Caption:=ProgramTitle;
  {read table}
  AssignFile(fin,'table.txt');
  try
    reset(fin);
  except
    ShowMessage('Table file access error');
    halt;
  end;
  Tmax:=0; {number of table element}
  while not eof(fin) do
  begin
    readln(fin,inst);
    if Tmax=0 then
    begin
      if copy(inst,1,3)<>#$EF#$BB#$BF then {UTF8 with BOM?}
      begin
        ShowMessage('Table file format error');
        CloseFile(fin);
        halt;
      end;
      inst:=copy(inst,4,100); {remove BOM}
    end;
    inc(Tmax);
    if Tmax>300 then
    begin
      ShowMessage('Too much elements on table.txt');
      halt;
    end;
    try
      i:=UTF8pos(#9,inst); {first tab}
      if i=0 then raise Exception.Create('Dummy');
      Table[Tmax].es:=UTF8copy(inst,1,i-1); {spanish part}
      UTF8delete(inst,1,i); {delete}
      i:=UTF8pos(#9,inst); {second tab}
      if i=0 then raise Exception.Create('Dummy');
      Table[Tmax].kana:=UTF8copy(inst,1,i-1); {kana part}
      UTF8delete(inst,1,i); {delete}
      Table[Tmax].size:=strtoint(inst); {size part}
    except
      ShowMessage('Invalid content in table.txt');
      halt;
    end;
  end;
  CloseFile(fin);
  if ParamCount=1 then  {open with file drop?}
  begin
    WorkFile:=ParamStr(1);
    FileLoad;
  end;
end;

procedure TForm1.FormDropFiles(Sender: TObject; const FileNames: array of String);
begin
  if TextChange then
    if Application.MessageBox('変更内容が失われます','確認',MB_OKCANCEL)=IDCANCEL then
       exit;
  WorkFile:=FileNames[0];
  FileLoad;
end;

procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
var
  result : integer;
begin
  if TextChange then
  begin
    result:=Application.MessageBox('内容をファイルに保存しますか？','確認',MB_YESNOCANCEL);
    if result=IDCANCEL then
    begin {cancel}
      CloseAction:=caNone;
      exit;
    end;
    if result=IDYES then MenuItem3Click(Sender); {save}
  end;
end;

procedure TForm1.MenuItem2Click(Sender: TObject); {open}
begin
  if TextChange then
    if Application.MessageBox('変更内容が失われます','確認',MB_OKCANCEL)=IDCANCEL then
       exit;
  if Open.Execute then
  begin
    if not FileExists(Open.FileName) then
    begin
      ShowMessage('ファイルがありません');
      exit;
    end;
    WorkFile:=Open.FileName;
    FileLoad;
  end;
end;

procedure TForm1.MenuItem3Click(Sender: TObject);  {save}
var
  f : text;
begin
  Save.InitialDir:=ExtractFilePath(WorkFile);
  Save.FileName:=ExtractFileName(WorkFile);
  if Save.Execute then
  begin
    if FileExists(Save.FileName) then
      if Application.MessageBox('上書き保存しますか？','確認',MB_YESNO)<>IDYES then exit;
    try
      WorkFile:=Save.FileName;
      AssignFile(f,WorkFile);
      Rewrite(f);
      write(f,#$EF#$BB#$BF); {BOM}
      write(f,Memo1.Text);
    except
      ShowMessage('ファイルアクセスエラー');
      exit;
    end;
    CloseFile(f);
    TextChange:=false;
    Form1.Caption:=ExtractFileName(WorkFile)+' - '+ProgramTitle;
  end;
end;

procedure TForm1.MenuItem4Click(Sender: TObject);  {exit}
begin
  Form1.Close;
end;

procedure TForm1.MenuItem6Click(Sender: TObject); {paste}
begin
  Memo1.Text:=Clipboard.AsText;
end;

procedure TForm1.MenuItem7Click(Sender: TObject); {copy}
begin
  Clipboard.AsText:=Memo1.SelText;
end;

procedure TForm1.MenuItem8Click(Sender: TObject); {select all}
begin
  Memo1.SelectAll;
end;

procedure TForm1.MenuItem9Click(Sender: TObject); {rubi}

function accent_red(s: UTF8string): UTF8string; {accent reduction}
var
  i,j : integer;
  st  : string;
begin
  st:=s;
  for i:=1 to UTF8Length(st) do
  begin
    j:=UTF8pos(UTF8copy(st,i,1),'áéíóú');
    if j>0 then
    begin
      UTF8delete(st,i,1);      {replace}
      UTF8insert(UTF8copy('aeiou',j,1),st,i);
    end;
  end;
  accent_red:=st;
end;

var
  buffer,
  inst,outst : string;
  i          : integer;
  found      : boolean;

begin
  buffer:=Memo1.Text+#13#10; {save to buffer}
  Memo1.Text:=''; {clear}
  repeat
    i:=pos(#13#10,buffer);
    inst:=copy(buffer,1,i-1); {get one line}
    delete(buffer,1,i+1); {delete line + CRLF}
    Memo1.Append(inst); {original test}
    if inst<>'' then
    begin
      inst:=UTF8lowerCase(inst); {lowercase}
      inst:=accent_red(inst);  {accent reduction}
      outst:=''; {clear}
      repeat
        found:=false;
        for i:=1 to Tmax do
        begin
          if UTF8copy(inst,1,UTF8length(Table[i].es))=Table[i].es then
          begin
            outst:=outst+Table[i].kana;  {furigana}
            UTF8delete(inst,1,Table[i].size); {eliminate the part}
            found:=true;
            break;
          end;
        end;
        if not found then
        begin
          outst:=outst+UTF8copy(inst,1,1);  {add 1 char}
          UTF8delete(inst,1,1); {eliminate 1 char}
        end;
      until UTF8length(inst)=0;
      Memo1.Append(outst);
    end;
  until buffer='';
end;

end.

